/*
 * @(#)NavigationComponent.java 11/6/2011
 *
 * Copyright 2002 - 2011 JIDE Software Inc. All rights reserved.
 */

package com.jidesoft.swing;

/**
 * A markup interface to indicate a component is for the navigation. All navigation components have
 * a consistent rollover and selection style.
 */
public interface NavigationComponent {
}
