/*
 * @(#) PopupUI.java
 * Last modified date: 2/9/2005
 *
 * Copyright 2002 - 2005 JIDE Software Inc. All rights reserved.
 */
package com.jidesoft.plaf;

import javax.swing.plaf.ComponentUI;
import java.awt.*;

/**
 * ComponentUI for PopupUI.
 */
public abstract class PopupUI extends ComponentUI {
    abstract public Component getGripper();
}
