package com.jidesoft.dialog;

/**
 * An interface to define commonly use button names.
 */
public interface ButtonNames {

    public static String APPLY = "APPLY";
    public static String BACK = "BACK";
    public static String BROWSE = "BROWSE";
    public static String CANCEL = "CANCEL";
    public static String CLEAR = "CLEAR";
    public static String CLOSE = "CLOSE";
    public static String DETAILS = "DETAILS";
    public static String EDIT = "EDIT";
    public static String EXIT = "EXIT";
    public static String FIND = "FIND";
    public static String FIND_NEXT = "FIND_NEXT";
    public static String FINISH = "FINISH";
    public static String FORWARD = "FORWARD";
    public static String HELP = "HELP";
    public static String NEW = "NEW";
    public static String NEXT = "NEXT";
    public static String NO = "NO";
    public static String OK = "OK";
    public static String OPEN = "OPEN";
    public static String PRINT = "PRINT";
    public static String REPLACE = "REPLACE";
    public static String RETRY = "RETRY";
    public static String RUN = "RUN";
    public static String SAVE = "SAVE";
    public static String SAVE_AS = "SAVE_AS";
    public static String STOP = "STOP";
    public static String YES = "YES";
    public static String RESET = "RESET";
}
