/*
 * @(#)WholeWordsSupport.java 2/6/2013
 *
 * Copyright 2002 - 2013 JIDE Software Inc. All rights reserved.
 */

package com.jidesoft.swing;

/**
 * An interface to support whole words.
 *
 * @since 3.5.2
 */
public interface WholeWordsSupport {
    public boolean isWholeWords();

    public void setWholeWords(boolean wholeWords);
}
