/*
 * @(#)VsnetCommandBarSeparatorUI.java
 *
 * Copyright 2002 JIDE Software. All rights reserved.
 */
package com.jidesoft.plaf.vsnet;

import com.jidesoft.plaf.basic.BasicGripperUI;

import javax.swing.*;
import javax.swing.plaf.ComponentUI;

/**
 */
public class VsnetGripperUI extends BasicGripperUI {
    public static ComponentUI createUI(JComponent c) {
        return new VsnetGripperUI();
    }

}
