/*
 * @(#)AbstractNumericRange.java
 * 
 * 2002 - 2012 JIDE Software Incorporated. All rights reserved.
 * Copyright (c) 2005 - 2012 Catalysoft Limited. All rights reserved.
 */

package com.jidesoft.range;

/**
 * An abstract class as the superclass of numeric ranges
 *
 * @author Simon White (swhite@catalysoft.com)
 */
public abstract class AbstractNumericRange<T> extends AbstractRange<T> {

    public AbstractNumericRange() {
        super();
    }

}
